/************************************************************************  
 *  File: hdlc.c - HDLC serial frame decoder/encoder
 *  Author: Mario Roessler, DH5YM
 *  Date: 2010/01/29
 *
 *
 ************************************************************************/

#include <stdlib.h>
#include <string.h>
#include <avr/io.h>
#include <avr/pgmspace.h>
#include <avr/interrupt.h>
//#include "lcd_1.h"
#include "uart.h"
//#include "kiss.h"
#include "hdlc.h"

/***************************************
** Globally needed variables definitions
 ***************************************/
uint8_t rxbyte;  //HDLC bit receiver collect byte
uint8_t rxbit=0;
#ifdef NRZI
  uint8_t rxbitNRZI=0;
#endif
uint8_t hdlcstatus=0; //status byte for the bit-decoder (0=ok,1=frame abort)
uint8_t rxbitc=0; //number of received bits
uint8_t rxone=0;  //number of consecutive ones
uint16_t rxbytec=0; //number of received bytes
uint8_t hdlcbyte=0;   //byte that stores the complete bytes
uint8_t framec=0; //number of received frames
uint16_t bytes=0; //number of bytes to transmit via kiss

char pRXbuf[MAXRXBUFFER]; //buffer for received frame


/**********************************************************
 * initialize timer and variables for collecting HDLC bytes
 **********************************************************/
void timer_init(void)
{

  //init timer 0 (not finished)
  TCCR0B = ( (1 << CS00) | (1 << CS01) ); // Teiler: 64
  //OCR0A = 249;       //1 tick every ms because 0.001s/(1/16e6Hz)/64
  OCR0A = 10;

  TCCR0A |= (1<<WGM01);
  TCCR0A &= ~(1<<WGM00);
  TCCR0B &= ~(1<<WGM02);  //clear timer on compare match

  #ifndef SOFTINT
  TIMSK0 = ( 1 << OCIE0A );         // Overflow Interrupt einschalten
  #endif

  return;
}


/***********************
 * init_rxhdlc
 * initialize the ports for RX HDLC
 ***********************/
void init_rxhdlc(void)
{ 

  DDR(RXC_PORT) &= ~_BV(RXC_PIN);
  DDR(RXD_PORT) &= ~_BV(RXD_PIN);
  DDR(DCD_PORT) &= ~_BV(DCD_PIN);

  //activate Interrupt for Int0 on rising edge
  EICRA |= ( 1<<ISC01 | 1<<ISC00 );

  #ifdef SOFTINT
  //activate Interrupt for Int1 (PD3) on rising edge
  DDRD |= (1<<PD3); //set PD3 to output in order to get a software interrupt
  PORTD &= ~(1<<PD3);  //reset the Pin
  EICRA |= ( 1<<ISC11 | 1<<ISC10 ); //interrupt on rising edge
  EIMSK |= (1<<INT1); //enable int1
  #endif

  //activate Pullup for DCD
  DCD_PORT |= (1<<DCD_PIN);

  //activate Pullup for RXClock
  //RXC_PORT |= (1<<RXC_PIN);
}


void start_rxhdlc(void)
{
  //activate Interrupt
  EIMSK |= (1<<INT0);
}

void stop_rxhdlc(void)
{
  //activate Interrupt
  EIMSK &= ~(1<<INT0);
}


/***************************************************
 * hdlc_getframe(void) / HDLC Bytewise framecollect
 ***************************************************/
void hdlc_getframe()
{
  if(hdlcstatus & 0x02) //flag detected
  {
    //frame complete or start
    if(rxbytec>7)  //a frame is just valid if it has 7 byte minimum (check if this is correct!!!)
	{
      bytes=rxbytec;
	  framec++;
	  //uart_puts("#FRMCMPL#");
	}
	hdlcstatus &= ~0x02; //reset flag detected flag
	rxbytec=0; //reset bytecount
	PORTA ^= (1<<PA0); //byte length debug
	return;
  } 

  if(hdlcstatus & 0x04) //one byte complete
  {
    //copy byte to buffer
	pRXbuf[rxbytec]=hdlcbyte;
    rxbytec++;
	hdlcstatus &= ~0x04; //reset byte complete flag
	PORTA ^= (1<<PA0); //byte length debug
  } else if(hdlcstatus & 0x01)
  {
    //frame abort, reset all and do not send via serial line
    rxbytec=0;
	hdlcstatus &= ~0x01; //reset frame abort flag
  } else 
  return;
}


/***************************************************
 * hdlc_printbit(void)
 * function for printing raw HDLC data to UART
 ***************************************************/
void hdlc_printbit(void)
{
  uint8_t rxstate = (PIN(RXD_PORT) & (1<<RXD_PIN));
  if (rxstate==0)
  {
    uart_putc('0');
  } else
  {
    uart_putc('1');
  }
}


/****************************************************************
 * ISR for Receive clock handling and sampling of the serial bits
 ****************************************************************/
ISR( INT0_vect ) {

  rxbit = (rxbit>>1);

  #ifdef NRZI
	rxbit |= (!(((PIN(NRZ_PORT) & (1<<NRZ_PIN)) >> NRZ_PIN) ^ ((PIN(RXD_PORT) & (1<<RXD_PIN)) >> RXD_PIN))) << 7;  //get bit out of information
	NRZ_PORT &= ~(1<<NRZ_PIN);  //reset NRZI storage Pin
	NRZ_PORT |= ((PIN(RXD_PORT) & (1<<RXD_PIN)) >> RXD_PIN) << NRZ_PIN;  // store modem input to NRZ pin
  #endif

  #ifdef NRZ
    rxbit |= ((PINC << 2) & 0x80); //shift in the current bit (LSB first)
  #endif

  #ifdef debug_hdlc
    if(rxbit & 0x80)
	{
      uart_puts("1");
	} else
	{
      uart_puts("0");
	}
  #endif
  if(rxbit==0x7E)  //flag
  {
    rxbitc=0;
	rxbit=0;
    hdlcstatus |= 0x02; //status flag detected
	#ifdef SOFTINT
	PORTD |= (1<<PD3); //create soft interrupt
    #endif
	#ifdef debug_hdlc
	  uart_puts("\n\r");
	#endif
  } else if((rxbit & 0xFE)==0xFE) //frame abort
  {
    rxbitc=0;
	rxbit=0;
	hdlcstatus |= 0x01; //status frame abort
	#ifdef SOFTINT
	PORTD |= (1<<PD3); //create soft interrupt
	#endif
  } else if((rxbit & 0x3F) == 0x1F)  //stuffing zero detected (rxbit = xx011111) => remove the 0
  {
    //nothing
  } else  //we received an ordinary byte and detected no exception
  {
    rxbyte = (rxbyte>>1) | ((rxbit << 2) & 0x80); //add the bit xx1xxxxx to the byte register
	rxbitc++; //increase bit counter
	if(rxbitc==10) //one byte completed is at 10 because we have to shift out the first 2 bits if we started after a flag
	{
	  hdlcbyte=rxbyte; //copy the byte
      hdlcstatus |= 0x04; //0x04 means byte complete
	  rxbitc=2;  //if one byte was received then the next 8 bit are also valid, no shift out of bits
	  rxbyte=0;
	  #ifdef SOFTINT
	  PORTD |= (1<<PD3); //create soft interrupt
	  #endif
	  #ifdef debug_hdlc
        uart_puts("-");
	  #endif
	}
  }
  return;
}


#ifdef SOFTINT
//ISR for bytecount (startet by bitcount routine)
ISR( INT1_vect ) 
{
  PORTD &= ~(1<<PD3); //reset interrupt at PD3
  #ifndef debug_hdlc
  hdlc_getframe();  //call hdlc_getframe
  #endif
  return;
}
#endif

#ifndef SOFTINT
//Timer0 COMPA ISR for cyclic start of byte assembling routine
ISR( TIMER0_COMPA_vect )
{
  #ifndef debug_hdlc
  hdlc_getframe();  //call hdlc_getframe every ms
  #endif

  return;
}
#endif

